from trelloapi import API_URL, AUTH_PARAMS
from cfg_authkeys import APP_KEY, AUTH_TOKEN
import requests
import json
import os.path
import sys

# Set of expected list names in an Eisenhower Decision Matrix
expected_list_names = set()
expected_list_names.add("important urgent")
expected_list_names.add("important not urgent")
expected_list_names.add("unimportant urgent")
expected_list_names.add("unimportant not urgent")
expected_list_names.add("done")

# Used to convert a list name to an abbreviated form for ease of use in CLI
list_name_convert = {
    "important urgent": "iu",
    "unimportant urgent": "uu",
    "important not urgent": "inu",
    "unimportant not urgent": "unu",
    "done": "done"
}

# Populated by getters to contain IDs for the lists in the Trello board
item_ids = {}


def set_existing_board_config():
    """ Creates a config file for an existing Trello board with the Eisenhower
    Decision Matrix categories. Populates the item_ids containing IDs for the
    items in the existing Trello board for external use.
    :return:
    """
    print "Go to your Trello board and get the board ID from the url\n" \
          "trello.com/b/[board ID]/[board name]"
    board_id = raw_input("Enter your Eisenhower Decision Matrix board ID: ")

    try:
        request = requests.get("%s/boards/%s/lists?%s" % (API_URL, board_id, AUTH_PARAMS))
        request.raise_for_status()
    except requests.exceptions.HTTPError as e:
        print e
        sys.exit(1)

    lists = json.loads(request.text)
    item_ids["board"] = lists[0]["idBoard"] # Uses long ID rather than shorthand

    for list in lists:
        if list["name"].lower() in expected_list_names:
            item_ids[list_name_convert[list["name"].lower()]] = list["id"]

    if len(item_ids) < 6: # 5 lists + board ID = 6
        print ("Invalid board. Your board must contain the following lists:"
               "\nImportant Urgent"
               "\nImportant Not Urgent"
               "\nUnimportant Urgent"
               "\nUnimportant Not Urgent"
               "\nDone")
        sys.exit(1)

    create_config_file()


def set_new_board_config():
    """ Creates a config file for a new Trello board with the Eisenhower
    Decision Matrix categories. Populates the item_ids containing IDs for the
    items in the new Trello board for external use.
    :return:
    """
    board_name = "Todo Matrix"
    board_desc = "A simple decision making tool"
    post_board_url = "%s/boards?name=%s&desc=%s&%s" % (API_URL, board_name, board_desc, AUTH_PARAMS)
    request = requests.post(post_board_url)

    response = json.loads(request.text)
    board_id = response["id"]
    item_ids["board"] = board_id
    empty_board(board_id)
    init_todo_matrix(board_id)
    create_config_file()


def empty_board(board_id):
    """ Archives all lists in a board.
    NOTE: When creating a board with Trello API, 3 default lists are created.
    However, when creating a board on the web app, default lists are not
    created. It's possible that this inconsistency is fixed in the future and
    thus this method won't be needed.
    :param board_id:
    :return:
    """
    get_lists_url = "%s/boards/%s/lists?%s" % (API_URL, board_id, AUTH_PARAMS)
    request = requests.get(get_lists_url)

    response = json.loads(request.text)
    for list in response:
        list_id = list["id"]
        put_list_url = "%s/lists/%s/closed?value=true&%s" % (API_URL, list_id, AUTH_PARAMS)
        requests.put(put_list_url)


def init_todo_matrix(board_id):
    """ Initializes the Todo Box board with the 4 Eisenhower Decision Matrix
    categories and a "Done" list
    :param board_id:
    :return:
    """
    done = requests.post("%s/boards/%s/lists?name=%s&%s" % (API_URL, board_id, "Done", AUTH_PARAMS))
    item_ids["done"] = json.loads(done.text)["id"]
    unu = requests.post("%s/boards/%s/lists?name=%s&%s" % (API_URL, board_id, "Unimportant Not Urgent", AUTH_PARAMS))
    item_ids["unu"] = json.loads(unu.text)["id"]
    uu = requests.post("%s/boards/%s/lists?name=%s&%s" % (API_URL, board_id, "Unimportant Urgent", AUTH_PARAMS))
    item_ids["uu"] = json.loads(uu.text)["id"]
    inu = requests.post("%s/boards/%s/lists?name=%s&%s" % (API_URL, board_id, "Important Not Urgent", AUTH_PARAMS))
    item_ids["inu"] = json.loads(inu.text)["id"]
    iu = requests.post("%s/boards/%s/lists?name=%s&%s" % (API_URL, board_id, "Important Urgent", AUTH_PARAMS))
    item_ids["iu"] = json.loads(iu.text)["id"]


def create_config_file():
    """ Takes all the list IDs and board ID from the item_ids and prints them
    out to a config file for use by the CLI script.
    :return:
    """
    configfile_name = "cfg_itemids.py"
    configfile_path = os.path.dirname(os.path.realpath(__file__))

    if os.path.isfile(os.path.join(configfile_path, configfile_name)):
        print "WARNING: This will rewrite your current Trello board's " \
              "configuration file."
        raw_input("Quit to stop the program or press Enter to continue...")

    ids = ("BOARD_ID = \"%s\""
           "\nIU_ID = \"%s\""
           "\nINU_ID = \"%s\""
           "\nUU_ID = \"%s\""
           "\nUNU_ID = \"%s\""
           "\nDONE_ID = \"%s\""
           % (item_ids["board"], item_ids["iu"], item_ids["inu"],
              item_ids["uu"], item_ids["unu"], item_ids["done"]))
    f = open(os.path.join(configfile_path, configfile_name), 'w')
    f.write(ids)
    print "Your Trello board is now hooked up to the CLI."


def create_auth_file():
    """ Creates a `cfg_authkeys.py` file by prompting the user for their Trello
    app key and auth token.
    :return:
    """
    if (APP_KEY != "") and (AUTH_TOKEN != ""):
        print "WARNING: This will rewrite the app key and auth token " \
              "already being used."
        raw_input("Quit to stop the program or press Enter to continue...")

    auth_filepath = os.path.dirname(os.path.realpath(__file__))

    app_key = raw_input("Enter your Trello app key: ")
    auth_token = raw_input("Enter your Trello auth token: ")
    authkeys = ("APP_KEY = \"%s\""
                "\nAUTH_TOKEN = \"%s\""
                % (app_key, auth_token))

    auth_filename = 'cfg_authkeys.py'
    f = open(os.path.join(auth_filepath, auth_filename), 'w')
    f.write(authkeys)

    print "Your Trello account is now hooked up to the CLI."



"""YoDoit

Usage:
    yodoit <card-name> [--desc <desc>] (--iu | --inu | --uu | --unu | --done)
    yodoit -l [<list-names>...]
    yodoit -o
    yodoit --authorize
    yodoit --set-new-board
    yodoit --set-existing-board

Options:
    --iu    Use Important Urgent list.
    --inu   Use Important Not Urgent list.
    --uu    Use Unimportant Urgent list.
    --unu   Use Unimportant Not Urgent list.
    --done  Use Done list.
    --desc  Add a description to a card.
    -l      Show an ordered list of cards in all lists, or a specified list.
    -o      Open Trello Board in browser.

Setting up the CLI:
    --authorize
            Prompts user to enter Trello app key and auth token.
    --set-new-board
            Sets a new board for the CLI.
    --set-existing-board
            Sets an existing board for the CLI by using board ID.
"""

from docopt import docopt
from trelloapi import API_URL, AUTH_PARAMS
import cfg_itemids as items
import getboardconfig
import json
import requests


item_ids = {
    'board': items.BOARD_ID,
    'iu': items.IU_ID,
    'inu': items.INU_ID,
    'uu': items.UU_ID,
    'unu': items.UNU_ID,
    'done': items.DONE_ID
}

name_list_convert = {v: k for k, v in getboardconfig.list_name_convert.items()}

def add_card_to_list(list_name, card_name, card_desc):
    if card_desc is None:
        card_desc = ''

    post_api = '%s/lists/%s/cards?name=%s&desc=%s&due=%s&%s' % (API_URL, item_ids[list_name], card_name, card_desc, 'null', AUTH_PARAMS)
    request = requests.post(post_api)

    if request.status_code == requests.codes.ok:
        print "Card \"" + card_name + "\" added to " + name_list_convert[list_name].title() + "."
    else:
        print "Failed."


def list_cards(list_names):
    if not list_names:
        list_names = ['done', 'iu', 'inu', 'uu', 'unu'] # Hard coded instead of using keys() on name_list_convert because it enforces order on output

    output = []

    for list_name in list_names:
        list_category = name_list_convert[list_name].title() + ":\n"
        get_api = '%s/lists/%s/cards?%s' % (API_URL, item_ids[list_name], AUTH_PARAMS)
        request = requests.get(get_api)
        cards = json.loads(request.text)

        i = 1
        for card in cards:
            list_category += `i` + ". " + card['name'] + "\n"
            i += 1

        output.append(list_category)

    s = "\n"
    print s.join(output)


def main():
    args = docopt(__doc__, version='1.0.5')

    if args['--iu']:
        add_card_to_list('iu', args['<card-name>'], args['<desc>'])
    elif args['--inu']:
        add_card_to_list('inu', args['<card-name>'], args['<desc>'])
    elif args['--uu']:
        add_card_to_list('uu', args['<card-name>'], args['<desc>'])
    elif args['--unu']:
        add_card_to_list('unu', args['<card-name>'], args['<desc>'])
    elif args['-l']:
        list_cards(args['<list-names>'])
    elif args['-o']:
        import webbrowser
        new = 2 # Open in new tab if browser already open
        url = "https://trello.com/b/%s" % item_ids['board']
        webbrowser.open(url, new)
    elif args['--authorize']:
        getboardconfig.create_auth_file()
    elif args['--set-new-board']:
        getboardconfig.set_new_board_config()
    elif args['--set-existing-board']:
        getboardconfig.set_existing_board_config()

if __name__ == '__main__':
    main()


from setuptools import setup

setup(
    name='yodoit',
    description='Python CLI for using an ' \
                'Eisenhower Decision Matrix with Trello',
    version='1.0.5',
    url='https://github.com/niketanpatel/yodoit-cli',
    author='Niketan Patel',
    scripts=['bin/yodoit'],
    packages=['yodoit'],
    license='MIT',
    keywords='todo list todolist trello developer ' \
             'cli eisenhower decision matrix box productivity method'
)